@@ -1,7 +1,7 @@
 #
 # linim.R
 #
-#  $Revision: 1.79 $   $Date: 2020/12/19 05:25:06 $
+#  $Revision: 1.81 $   $Date: 2022/06/19 00:53:04 $
 #
 #  Image/function on a linear network
 #
@@ -367,13 +367,14 @@ as.linim <- function(X, ...) {
 }
 
 as.linim.default <- function(X, L, ..., eps = NULL, dimyx = NULL, xy = NULL,
-                                        delta = NULL) {
+                                        delta = NULL, nd = NULL) {
   stopifnot(inherits(L, "linnet"))
   Y <- as.im(X, W=Frame(L), ..., eps=eps, dimyx=dimyx, xy=xy)
   M <- as.mask.psp(as.psp(L), as.owin(Y))
   Y[complement.owin(M)] <- NA
   df <- NULL
-  if(!is.null(delta)) {
+  if(!is.null(delta) || !is.null(nd)) {
+    if(is.null(delta)) delta <- volume(L)/nd
     df <- pointsAlongNetwork(L, delta)
     pix <- nearest.valid.pixel(df$x, df$y, Y)
     df$xc <- Y$xcol[pix$col]
@@ -388,34 +389,29 @@ as.linim.default <- function(X, L, ..., eps = NULL, dimyx = NULL, xy = NULL,
   return(out)
 }
 
-pointsAlongNetwork <- local({
-
-  pointsAlongNetwork <- function(L, delta) {
-    #' sample points evenly spaced along each segment
-    stopifnot(inherits(L, "linnet"))
-    S <- as.psp(L)
-    ns <- nsegments(S)
-    seglen <- lengths_psp(S)
-    ends <- as.data.frame(S)
-    nsample <- pmax(1, ceiling(seglen/delta))
-    df <- NULL
-    x0 <- ends$x0
-    y0 <- ends$y0
-    x1 <- ends$x1
-    y1 <- ends$y1
-    for(i in seq_len(ns)) {
-      nn <- nsample[i] + 1L
-      tcut <- seq(0, 1, length.out=nn)
-      tp <- (tcut[-1] + tcut[-nn])/2
-      x <- x0[i] * (1-tp) + x1[i] * tp
-      y <- y0[i] * (1-tp) + y1[i] * tp
-      df <- rbind(df, data.frame(x=x, y=y, seg=i, tp=tp))
-    }
-    return(df)          
+pointsAlongNetwork <- function(L, delta) {
+  #' sample points evenly spaced along each segment
+  stopifnot(inherits(L, "linnet"))
+  S <- as.psp(L)
+  ns <- nsegments(S)
+  seglen <- lengths_psp(S)
+  ends <- as.data.frame(S)
+  nsample <- pmax(1, ceiling(seglen/delta))
+  df <- NULL
+  x0 <- ends$x0
+  y0 <- ends$y0
+  x1 <- ends$x1
+  y1 <- ends$y1
+  for(i in seq_len(ns)) {
+    nn <- nsample[i] + 1L
+    tcut <- seq(0, 1, length.out=nn)
+    tp <- (tcut[-1] + tcut[-nn])/2
+    x <- x0[i] * (1-tp) + x1[i] * tp
+    y <- y0[i] * (1-tp) + y1[i] * tp
+    df <- rbind(df, data.frame(x=x, y=y, seg=i, tp=tp))
   }
-
-  pointsAlongNetwork
-})
+  return(df)          
+}
 
 as.linim.linim <- function(X, ...) {
   if(length(list(...)) == 0)
@@ -604,21 +600,30 @@ as.linnet.linim <- function(X, ...) {
   return(y)
 }
 
-integral.linim <- function(f, domain=NULL, ...){
+integral.linim <- function(f, domain=NULL, weight=NULL, ...){
   verifyclass(f, "linim")
+  if(!is.null(weight))
+    weight <- as.linim(weight, domain(f))
   if(is.tess(domain)) {
     result <- sapply(tiles(domain), integral.linim, f = f)
     if(length(dim(result)) > 1) result <- t(result)
     return(result)
   }
-  if(!is.null(domain)) 
+  if(!is.null(domain)) {
     f <- f[domain]
+    if(!is.null(weight)) weight <- weight[domain]
+  }
   #' extract data
   L <- as.linnet(f)
   ns <- nsegments(L)
   df <- attr(f, "df")
   vals <- df$values
   seg <- factor(df$mapXY, levels=1:ns)
+  #' weight
+  if(!is.null(weight)) {
+    samplepoints <- ppp(df$xc, df$yc, window=Frame(L), check=FALSE)
+    vals <- vals * safelookup(weight, samplepoints)
+  }
   #' ensure each segment has at least one sample point
   nper <- table(seg)
   if(any(missed <- (nper == 0))) {
@@ -626,6 +631,8 @@ integral.linim <- function(f, domain=NULL, ...){
     mp <- midpoints.psp(as.psp(L)[missed])
     #' nearest pixel value
     valmid <- safelookup(f, mp)
+    if(!is.null(weight))
+      valmid <- valmid * safelookup(weight, mp)
     #' concatenate factors
     seg <- unlist(list(seg, factor(missed, levels=1:ns)))
     vals <- c(vals, valmid)
